/// @file           matrixkeypad.c
/// @copyright      Copyright (c) 2014 Toradex AG.
///                 [Software License Agreement]
/// $Author:        khv
/// $Date:          $ 2014-08-12 $
/// @brief:         File to support matrix keypad interfacing.

/// Header files.
#include "matrixkeypad.h"
#include "gpio.h"

/// Declarations.
static BOOL keypadConfigured = FALSE;
static DWORD rowPin[] = GET_ROW;
static DWORD colPin[] = GET_COLUMN;
static uIo ioPin = {0};
volatile BOOL threadExit = FALSE;
 
/// Functions prototype.

//******************************************************************************
/// Sets a particular column as High.
/// @param[in]   hGpio              Handle of GPIO library instance.
/// @param[in]   columnStateHigh    Column to set as HIGH.
void SelectCol(HANDLE hGpio, DWORD column);

//******************************************************************************
/// Scans for a row that is High.
/// @param[in]   hGpio    Handle of GPIO library instance.
/// @param[out]  row      Row that is at HIGH level.
/// @retval      TRUE     Success, detected a key-press.
///              FALSE    Failure, no key-press detected.
BOOL GetRow(HANDLE hGpio, DWORD *row);

//******************************************************************************
/// Configures the GPIOs, used as input and output.
BOOL MatrixKeypadConfig(HANDLE hGpio, tIoType ioType)
{
    DWORD rowCount = 0;
    DWORD columnCount = 0;
    BOOL retVal = FALSE;

    if (keypadConfigured == FALSE)
    {
        ioPin.Type = ioType;
        /// Configures row pins as GPIO input.
        for (rowCount = 0; rowCount < TOTAL_ROWS; rowCount++)
        {
            ioPin.Number = rowPin[rowCount];
            /// Sets ioPin as GPIO.
            Gpio_ConfigureAsGpio(hGpio, ioPin);
            Gpio_SetDir(hGpio, ioPin, ioInput);
        }
        /// Configures column pins as GPIO output logic LOW.
        for (columnCount = 0; columnCount < TOTAL_COLUMNS; columnCount++)
        {
            ioPin.Number = colPin[columnCount];
            /// Sets column pin as GPIO output.
            retVal = Gpio_SetConfigString(hGpio, ioPin, NULL, L"altfn=-1,dir=out", StoreVolatile);
            if (retVal == FALSE)
            {
                return FALSE;
            }
            /// Sets level of the pin as LOW.
            Gpio_SetLevel(hGpio, ioPin, ioLow);
        }
        keypadConfigured = TRUE;
    }
    return TRUE;
}
 
//******************************************************************************
/// Scans for the key-press on matrix keypad. MatrixKeypadConfig() must be called before this.
BOOL MatrixKeypadScan(HANDLE hGpio)
{
    BOOL retVal = 0;
    DWORD columnCount = 0;
    DWORD row = 0;

    if (keypadConfigured == FALSE)
    {
        return FALSE;
    }
    for(columnCount = 0; threadExit == FALSE; columnCount++)
    {
        if (columnCount == TOTAL_COLUMNS)
        {
            columnCount = 0;
        }
        SelectCol(hGpio, columnCount);   ///< Sets column as High.
        retVal = GetRow(hGpio, &row);    ///< Scans for row that is High.
        if (retVal == TRUE)
        {
            /// Shows result after increasing index by 1 in both row and column.
            printf("\nKey pressed. Row number = %d, Column number = %d", row + 1, columnCount + 1);
        }
        Sleep(AUTO_REPEAT_TIMEOUT_PER_COLUMN);
    }
    return TRUE;
}
 
//******************************************************************************
/// Sets a particular column as High.
void SelectCol(HANDLE hGpio, DWORD columnStateHigh)
{
    DWORD columnStateLow = 0;
    
    columnStateLow = columnStateHigh;
    if (columnStateLow != 0)
    {
        columnStateLow--;
    }
    else
    {
        columnStateLow = TOTAL_COLUMNS - 1;
    }
    ioPin.Number = colPin[columnStateLow];
    /// Sets the previous column as LOW.
    Gpio_SetLevel(hGpio, ioPin, ioLow);
    ioPin.Number = colPin[columnStateHigh];
    /// Sets the next column as HIGH.
    Gpio_SetLevel(hGpio, ioPin, ioHigh);
}
 
//******************************************************************************
/// Scans for a row that is High.
BOOL GetRow(HANDLE hGpio, DWORD *row)
{
    DWORD rowCount = 0;
    BOOL retVal = FALSE;
    *row = 0;
    for (rowCount = 0; rowCount < TOTAL_ROWS; rowCount++)
    {
        ioPin.Number = rowPin[rowCount];
        /// Gets the level of the pin configured as row.
        retVal = Gpio_GetLevel(hGpio, ioPin);
        if (retVal == TRUE)
        {
            /// Time-out in ms to ignore the bouncing of the keystroke.
            Sleep(KEY_STABLE_HIT_TIMEOUT);
            /// Gets the level of the pin configured as row.
            retVal = Gpio_GetLevel(hGpio, ioPin);
            if (retVal == TRUE)
            {
                *row = rowCount;
                break;
            }
        }
    }
    return retVal;
}
